#############
# $Id: bam.pm 13905 2011-08-15 09:52:41Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2010-01-22
# Last Changed On: $Date: 2011-08-15 10:52:41 +0100 (Mon, 15 Aug 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-29.2/lib/npg_pipeline/archive/file/generation/bam.pm $

package npg_pipeline::archive::file::generation::bam;
use Moose;
use Carp;
use English qw{-no_match_vars};
use Readonly;
use List::MoreUtils qw{any};

extends q{npg_pipeline::archive::file::generation};

with qw{
  npg_common::roles::run::lane::tag_info
  npg_pipeline::roles::business::pre_exec_strings
};

Readonly::Scalar our $VERSION  => do { my ($r) = q$LastChangedRevision: 13905 $ =~ /(\d+)/mxs; $r; };

# these have been found to be the best for this process, so have been hard coded
Readonly::Scalar our $DEFAULT_NUMBER_OF_PROCESSORS => 4;

sub generate {
  my ( $self, $arg_refs ) = @_;

  my $id_run = $self->id_run();

  my $second_call_qseq_dir = $self->bustard_path() . q{/SecondCall};
  if ( ! $self->no_secondcall() && -e $second_call_qseq_dir ) {
    $arg_refs->{second_call_qseq_dir} = $second_call_qseq_dir;
  }

  $arg_refs->{read_number_1} = $self->sanger_read_one();
  if ( $self->is_paired_read() ) {
    $arg_refs->{read_number_2} = $self->sanger_read_two();
    if ( $self->is_indexed() ) {
      $arg_refs->{read_number_2} = $self->illumina_read_two();
    }
  }

  $self->log( qq{Creating Jobs to create bam files for $id_run} );

  my @positions = $self->positions( $arg_refs );

  my @job_ids;

  my $href = $self->lanes_by_split_nonsplit( @positions );

  my @bsub_commands;

  if ( scalar @{ $href->{split_nc_lanes} } ) {
    push @bsub_commands, $self->_generate_bsub_commands( {
      positions => $href->{split_nc_lanes},
      indexed_positions => $href->{indexed_lanes},
      lanes_by_type => $href,
      multiplex => 0,
      split_nc => 1,
      arg_refs => $arg_refs,
    } );
  }

  if ( scalar @{ $href->{non_split_nc_lanes} } ) {
    push @bsub_commands, $self->_generate_bsub_commands( {
      positions => $href->{non_split_nc_lanes},
      indexed_positions => $href->{indexed_lanes},
      lanes_by_type => $href,
      multiplex => 0,
      split_nc => 0,
      arg_refs => $arg_refs,
    } );
  }

  if ( scalar @{ $href->{spiked_phix_lanes} } ) {

    push @bsub_commands, $self->_generate_bsub_commands( {
      positions => $href->{spiked_phix_lanes},
      indexed_positions => $href->{indexed_lanes},
      lanes_by_type => $href,
      multiplex => 0,
      split_nc => 0,
      spiked_phix => 1,
      arg_refs => $arg_refs,
    } );
  }

  if ( scalar @{ $href->{split_nc_mp_lanes} } ) {
    push @bsub_commands, $self->_generate_bsub_commands( {
      positions => $href->{split_nc_mp_lanes},
      lanes_by_type => $href,
      multiplex => 1,
      split_nc => 1,
      arg_refs => $arg_refs,
    } );
  }

  if ( scalar @{ $href->{non_split_nc_mp_lanes} } ) {
    push @bsub_commands, $self->_generate_bsub_commands( {
      positions => $href->{non_split_nc_mp_lanes},
      lanes_by_type => $href,
      multiplex => 1,
      split_nc => 0,
      arg_refs => $arg_refs,
    } );
  }

  foreach my $cmd ( @bsub_commands ) {
    push @job_ids, $self->submit_bsub_command( $cmd );
  }


  return @job_ids;
}

###############
# responsible for generating the bsub commands to be executed
sub _generate_bsub_commands { ## no critic (Subroutines::ProhibitExcessComplexity)
  my ( $self, $outer_arg_refs ) = @_;

  my @bsub_commands;

  my $arg_refs = $outer_arg_refs->{arg_refs};

  my $positions = $outer_arg_refs->{positions};
  my $multiplex = $outer_arg_refs->{multiplex};
  my $split_nc  = $outer_arg_refs->{split_nc};
  my $spiked_phix  = $outer_arg_refs->{spiked_phix};

  my $fastq_files             = $arg_refs->{fastq_files};
  my $required_job_completion = $arg_refs->{required_job_completion};
  my $second_call_qseq_dir    = $arg_refs->{second_call_qseq_dir};
  my $read_number_1           = $arg_refs->{read_number_1};
  my $read_number_2           = $arg_refs->{read_number_2};

  my $id_run                  = $self->id_run();

foreach my $lane_type_recal ( qw(recalibrated_lanes non_recalibrated_lanes) ) {

  my $lanes_recal = $outer_arg_refs->{lanes_by_type}->{$lane_type_recal};

  next if ( ! scalar @{ $lanes_recal } );

  my %exists_lane_type = map { $_ => 1 } @{ $lanes_recal };

  if ( ! $multiplex && ! $spiked_phix ) {
    foreach my $position ( @{ $lanes_recal } ) {

      if ( any { $position == $_ } @{ $outer_arg_refs->{indexed_positions} } ) {
        delete $exists_lane_type{$position};
      }

    }
  }

  my @wanted_positions = grep { $exists_lane_type{$_} } @{$positions} ;
  next if ( ! scalar @wanted_positions  );
  $self->clear_lane_tag_pairs();

  my $output_path = $self->archive_path() . q{/};

  my $lsf_array_string;
  my $bam_filename = $id_run . q{_};

  if ( $multiplex ) {

    foreach my $position ( @wanted_positions ) {
      $self->add_lane_tag_pairs( $position );
    }

    $lsf_array_string = $self->lsf_job_array_from_lane_tag_pairs();
    $output_path .= q{lane} . $self->position_decode_string() . q{/};
    $bam_filename .=  $self->position_decode_string();

  } else {

    $lsf_array_string = $self->create_array_string( @wanted_positions );
    $bam_filename .= $self->lsb_jobindex();

  }

  my $script_bin = $self->internal_script_bin();

  my $log_folder = $self->make_log_dir( $self->archive_path() );

  my $outfile = $log_folder . q{/} . $self->bsub_output_file();

  my $job_name = $self->external_script_names_conf()->{bam_creator_script} . q{_} . $id_run . q{_} . $self->timestamp();

  my $out_subs_name = $job_name . q{.%I.%J};
  $outfile =~ s/substitute_this/$out_subs_name/xms;

  $job_name .= $lsf_array_string;
  if ( ! $self->no_array_cpu_limit() ) {
    $job_name .= q{%} . $self->array_cpu_limit();
  }
  $job_name = q{'} . $job_name . q{'};

  my @types;
  if ( $split_nc ) {
    @types = ( qw(human nonhuman) );
  }elsif( $spiked_phix ) {
    @types = ( qw{phix} );
  }else {
    @types = ( qw{none} );
  }

  $bam_filename = $output_path . $bam_filename;
  foreach my $type ( @types ) {

    my $specific_bam_filename = $bam_filename;

    if ( $type ne q{none} ) {
      $specific_bam_filename .= q{_} . $type;
    }

    if ( $multiplex ) {
      $specific_bam_filename .= q{#} . $self->tag_index_decode_string();
    }

    my $job_command = qq{$script_bin/} . $self->external_script_names_conf()->{bam_creator_script} . qq{ --id_run $id_run --position };
    if ( $multiplex ) {
      $job_command .= $self->position_decode_string();
      $job_command .= q{ --tag_index } . $self->tag_index_decode_string();
    } else {
      $job_command .= $self->lsb_jobindex();
    }

    $job_command .= qq{ --bam_filename $specific_bam_filename};

    if ( $type ne q{none} ) {
      if ( $type eq q{human} ) {
        $job_command .= q{ --reference } . $self->_get_reference({aligner=>$self->references_conf()->{human_split_aligner}, species => q{Homo_sapiens},});
        $job_command .= q{ --reference_fai } . $self->_get_reference({aligner=>q{fasta}, species => q{Homo_sapiens},}) . q{.fai};
      }elsif( $type eq q{phix} ){
         $job_command .= q{ --reference } . $self->_get_reference({aligner=>$self->references_conf()->{phix_split_aligner}, species => q{PhiX},});
         $job_command .= q{ --reference_fai } . $self->_get_reference({aligner=>q{fasta}, species => q{PhiX},}) . q{.fai};
      }
      $job_command .= qq{ --human_split $type};
    }

    if ( $second_call_qseq_dir ) {
      $job_command .= qq{ --second_call_qseq_dir $second_call_qseq_dir};
    }
    if ( $read_number_1 ) {
      $job_command .= qq{ --read_number_1 $read_number_1};
    }
    if ( $read_number_2 ) {
      $job_command .= qq{ --read_number_2 $read_number_2};
    }

    if( $lane_type_recal eq q{recalibrated_lanes} ){
       $job_command .= q{ --original_quality_qseq_dir } . $self->bustard_path();
    }

    $job_command .= q{ --archive_path } . $self->archive_path();

    # construct the fastq filenames that will be required
    my $fastq_file_base = $id_run . q{_};

    if ( $multiplex ) {
      $fastq_file_base = q{lane} . $self->position_decode_string() . q{/} . $fastq_file_base;
      $fastq_file_base .= $self->position_decode_string();
    } else {
      $fastq_file_base .= $self->lsb_jobindex();
    }

    if ( $self->is_paired_read() or $self->is_indexed() ) {
      $fastq_file_base .= q{_read};
    }

    if ( $type ne q{none} ) {
      $fastq_file_base .= q{_} . $type;
    }

    if ( $multiplex ) {
      $fastq_file_base .= q{#} . $self->tag_index_decode_string();
    }

    $fastq_file_base .= q{.fastq};

    my $fq1 = $fastq_file_base;
    $fq1 =~ s/read/1/xms;

    $job_command .= q{ --fastq_files } . $self->archive_path() . q{/} . $fq1;
    if ( $read_number_2 ) {
      my $fq2 = $fastq_file_base;
      $fq2 =~ s/read/2/xms;
      $job_command .= q{ --fastq_files } . $self->archive_path() . q{/}  . $fq2;
    }

    if ( $self->is_indexed() ) {
       my $p;
       if( $multiplex ){
         $p = $self->position_decode_string();
       }else{
         $p = $self->lsb_jobindex();
       }
       $job_command .= q{ --index_fastq_file } . $self->archive_path() . q{/} . $id_run . q{_} . $p . q{_t};
       if ( $type ne q{none} ) {
         $job_command .= q{_} . $type;
       }
       $job_command .= q{.fastq};
    }

    my $loop_job_name = $job_name;
    my $loop_outfile = $outfile;

    if ( $type ne q{none} ) {
      $loop_job_name =~ s/_{$id_run}_/_{$id_run}_{$type}_/xms;
      $loop_outfile  =~ s/_{$id_run}_/_{$id_run}_{$type}_/xms;
    }

    if ( $multiplex ) {
      $loop_job_name =~ s/_{$id_run}_/_{$id_run}_mp_/xms;
      $loop_outfile  =~ s/_{$id_run}_/_{$id_run}_mp_/xms;
    }

    my $resources = ( $self->fs_resource_string( {
      counter_slots_per_job => $DEFAULT_NUMBER_OF_PROCESSORS,
      resource_string => $self->_default_resources(),
    } ) );


    my $job_sub =  q{bsub -q } . $self->lsf_queue()
                .  q{ } . $self->ref_adapter_pre_exec_string()
                . qq{ $resources $required_job_completion -J $loop_job_name -o $loop_outfile '$job_command'};

    if ( $self->verbose() ) { $self->log($job_sub); }

    push @bsub_commands, $job_sub;
  }
}
  return @bsub_commands;
}

sub _default_resources {
  my ( $self ) = @_;
  my $mem = $self->general_values_conf()->{bam_creation_memory};
  return qq{-R 'select[mem>$mem] rusage[mem=$mem] span[hosts=1]' -M${mem}000 -n} . $DEFAULT_NUMBER_OF_PROCESSORS;
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 NAME

npg_pipeline::archive::file::generation::bam

=head1 VERSION

$LastChangedRevision: 13905 $

=head1 SYNOPSIS

  my $oAfgfq = npg_pipeline::archive::file::generation::bam->new({
    run_folder => $sRunFolder,
  });

=head1 DESCRIPTION

Object module which knows how to construct and submits the command line to LSF for creating bam file.

=head1 SUBROUTINES/METHODS

=head2 generate - generates the bsub jobs and submits them for creating the fastq files, returning an array of job_ids.

  my @job_ids = $oAfgfq->generate({
    required_job_completion} => q{-w (123 && 321)};
    timestamp                => q{20090709-123456};
    solexa_bin               => q{/software/solexa/bin};
    id_run                   => 1234;
    BSUB_ERROR_FILE          => q{substitute_this.err};
    BSUB_OUTPUT_FILE         => q{substitute_this.out};
  });

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Carp

=item English -no_match_vars

=item Readonly

=item Moose

=item npg_pipeline::archive::file::generation

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Guoying Qi (gq1@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
