#############
# $Id: illumina_analysis.pm 13905 2011-08-15 09:52:41Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2009-09-01
# Last Changed On: $Date: 2011-08-15 10:52:41 +0100 (Mon, 15 Aug 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-29.2/lib/npg_pipeline/archive/qc/illumina_analysis.pm $

package npg_pipeline::archive::qc::illumina_analysis;
use Moose;
use Carp;
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 13905 $ =~ /(\d+)/mxs; $r; };

extends qw{npg_pipeline::archive};

sub submit_to_lsf {
  my ($self, $arg_refs) = @_;
  my $job_sub = $self->_generate_bsub_command($arg_refs);
  my $job_id = $self->submit_bsub_command($job_sub);
  return ($job_id);
}

# private methods

sub _generate_bsub_command {
  my ($self, $arg_refs) = @_;

  my $required_job_completion = $arg_refs->{required_job_completion};
  my $timestamp = $self->timestamp();

  my $job_name = $self->external_script_names_conf()->{illumina_analysis_qc_loader} . q{_} . $self->id_run() . q{_} . $timestamp;

  my $options = $self->attributes_as_command_options({
    excluded_attributes => [qw(
        time timestamp lsf_queue script_bin run batch default_usr_bin
        summary_link max_lane owning_group mail_from control_lane_position default_control_lane internal_script_bin
        bsub_error_file bsub_output_file subpath log_file log_file_name log_file_path ARGV script_name array_cpu_limit
        conf_path domain
      )],
  });

  my $location_of_logs = $self->make_log_dir( $self->recalibrated_path() );
  my $bsub_command = q{bsub -q } . $self->lsf_queue() . qq{ $required_job_completion -J $job_name };
  $bsub_command .=  ( $self->fs_resource_string( {
    counter_slots_per_job => 1,
  } ) ) . q{ };
  $bsub_command .=  q{-E '} . $self->internal_script_bin() . q{/} . $self->_unique_script_runner() . q{ -job_name="} . $self->external_script_names_conf()->{illumina_analysis_qc_loader} . qq{" -own_job_name="$job_name"' };
  $bsub_command .=  q{-o } . $location_of_logs . q{/} . $job_name . q{.out };
  $bsub_command .=  q{'} . $self->internal_script_bin() . q{/} . $self->external_script_names_conf()->{illumina_analysis_qc_loader} . q{ };
  $bsub_command .=  qq{$options'};
  if ($self->verbose()) {
    $self->log($bsub_command);
  }
  return $bsub_command;
}


no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 NAME

npg_pipeline::archive::qc::illumina_analysis

=head1 VERSION

$LastChangedRevision: 13905 $

=head1 SYNOPSIS

  my $aia = npg_pipeline::archive::qc::illumina_analysis->new({
    run_folder => <run_folder>,
  });

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 submit_to_lsf - handles calling out to create the bsub command and submits it, returning the job ids

  my @job_ids = $aia->submit_to_lsf({
    required_job_completion => <lsf job requirement string>,
    timestamp => <timestamp string>,
  });

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item Readonly

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
