#############
# $Id: evaluator.pm 13905 2011-08-15 09:52:41Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2009-08-10
# Last Changed On: $Date: 2011-08-15 10:52:41 +0100 (Mon, 15 Aug 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-29.2/lib/npg_pipeline/dispatch_tree/evaluator.pm $

package npg_pipeline::dispatch_tree::evaluator;
use Moose;
use Carp;
use English qw{-no_match_vars};
use Perl6::Slurp;
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 13905 $ =~ /(\d+)/mxs; $r; };

extends qw{npg_pipeline::dispatch_tree};

$ENV{TEST_BIN} ||= q{};
$ENV{TEST_DIR} ||= q{};

has q{log_file}             => (isa => q{Str}, is => q{ro}, lazy_build => 1);
has q{+json_structure}      => (lazy => 1, builder => q{_build_json_structure});
has q{_functions_by_job_id} => (isa => q{HashRef}, is => q{ro}, lazy_build => 1);

sub evaluate_tree {
  my ($self) = @_;

  $self->_populate_with_json();
  my $functions_by_job_id = $self->_functions_by_job_id();
  my %statuses;

  foreach my $jid (sort keys %{$functions_by_job_id}) {
    my $bjob_info = `$ENV{TEST_BIN}bjobs $jid`;
    if ($bjob_info =~ /PEND/xms) {
      push @{$statuses{PEND}}, [q{PEND}, $functions_by_job_id->{$jid}->{function}, $jid, $functions_by_job_id->{$jid}->{job_dependencies}];
      next;
    }
    if ($bjob_info =~ /RUN/xms) {
      push @{$statuses{RUN}}, [q{RUN}, $functions_by_job_id->{$jid}->{function}, $jid, $functions_by_job_id->{$jid}->{job_dependencies}];
      next;
    }
    if ($bjob_info =~ /EXIT/xms) {
      push @{$statuses{EXIT}}, [q{EXIT}, $functions_by_job_id->{$jid}->{function}, $jid, $functions_by_job_id->{$jid}->{job_dependencies}];
      next;
    }
    if ($bjob_info =~ /DONE/xms) {
      push @{$statuses{DONE}}, [q{DONE}, $functions_by_job_id->{$jid}->{function}, $jid, $functions_by_job_id->{$jid}->{job_dependencies}];
      next;
    }
    if ($bjob_info =~ /Job[ ][<]$jid[>][ ]is[ ]not[ ]found/xms) {
      my $bhist_info = `$ENV{TEST_BIN}bhist -l $jid`;
      if ($bhist_info =~ /Exited[ ]with[ ]exit[ ]code/xms) {
        push @{$statuses{EXIT}}, [q{EXIT}, $functions_by_job_id->{$jid}->{function}, $jid, $functions_by_job_id->{$jid}->{job_dependencies}];
      } else {
        push @{$statuses{DONE}}, [q{DONE}, $functions_by_job_id->{$jid}->{function}, $jid, $functions_by_job_id->{$jid}->{job_dependencies}];
      }
    }
  }

  print qq{STATUS\tFunction\tJob ID\tDependencies\n} || carp qq{STATUS\tFunction\tJob ID\tDependencies\n};
  foreach my $status (qw{EXIT DONE RUN PEND}) {
    foreach my $row (@{$statuses{$status}}) {
      print join (qq{\t}, @{$row}) . qq{\n} || carp join qq{\t}, @{$row} . qq{\n};
    }
  }
  print qq{\n\n} || carp qq{\n\n};

  return 1;
}

sub _build_log_file {
  my ($self) = @_;

  if (!($self->has_id_run() || $self->has_run_folder())) {
    croak q{not enough information provided to work out log_file};
  }

  my $ls_cmd = q{ls } . $self->runfolder_path() . q{/log_} . $self->script_name().q{_*.log};
  my @log_files = `$ls_cmd`;

  if (!scalar@log_files) {
    croak q{no log files found};
  }

  @log_files = reverse sort @log_files;
  my $log_file = shift @log_files;

  chomp $log_file;

  return $log_file;
}

sub _build_json_structure {
  my ($self) = @_;

  my $file_content = slurp ($self->log_file());
  my ($json_content) = $file_content =~ /JSON[ ]Dispatch[ ]Tree:\n\n(.*?)\n\n/xms;
  return $json_content;
}

sub _build__functions_by_job_id {
  my ($self) = @_;
  my $href = {};
  foreach my $function (@{$self->functions()}) {
    $function->{job_dep_array} = [];
    my $dependencies = $function->{job_dependencies};
    my @jids = split /[^\d]+/xms, $function->{job_dependencies};
    foreach my $jid (@jids) {
      if ($jid) {
        push @{$function->{job_dependency_array}}, $jid;
      }
    }
    foreach my $jid (@{$function->{job_ids_launched}}) {
      ($jid) = $jid =~ /(\d+)/xms;
      $href->{$jid} = $function;
    }
  }
  return $href;
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 NAME

npg_pipeline::dispatch_tree::evaluator

=head1 VERSION

$LastChangedRevision: 13905 $

=head1 SYNOPSIS

  my $oDTE = npg_pipeline::dispatch_tree::evaluator->new({
    json_structure => $sJSON_STRUCTURE,
  });

  my $oDTE = npg_pipeline::dispatch_tree::evaluator->new({
    run_folder  => q{123456_IL2_1234},
    id_run      => q{1234},
    script_name => q{script_name},
  });

=head1 DESCRIPTION

Object module to determine how a pipeline has passed through LSF and outputs the statuses of the jobs

=head1 SUBROUTINES/METHODS

=head2 evaluate_tree - method which will take a json string of a dispatch tree, finding in a log file if provided and then outputting to STDOUT a list of the jobs and an evaluation of the status off each from LSF

  $oDTE->evaluate_tree();

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item Readonly

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
