#############
# $Id: post_qc_review.pm 13905 2011-08-15 09:52:41Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2009-11-05
# Last Changed On: $Date: 2011-08-15 10:52:41 +0100 (Mon, 15 Aug 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-29.2/lib/npg_pipeline/pluggable/harold/post_qc_review.pm $

package npg_pipeline::pluggable::harold::post_qc_review;
use Moose;
use Carp;
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 13905 $ =~ /(\d+)/mxs; $r; };

extends qw{npg_pipeline::pluggable::harold};

# this provides an in-built function order for post-qc_review should none be provided

sub _default_function_order {
  my ($self) = @_;
  return $self->function_order_conf()->{post_qc_review};
}

=head1 NAME

npg_pipeline::pluggable::harold::post_qc_review

=head1 VERSION

$LastChangedRevision: 13905 $

=head1 SYNOPSIS

  my $oPostQCReview = npg_pipeline::pluggable::harold::post_qc_review->new();

=head1 DESCRIPTION

Pluggable pipeline module for the post_qc_review pipeline

=head1 SUBROUTINES/METHODS

=head2 archive_to_irods

upload all archival files to irods

=cut

sub archive_to_irods {
  my ($self, @args) = @_;
  my $required_job_completion = shift @args;
  my $ats = $self->new_with_cloned_attributes(q{npg_pipeline::archive::file::to_irods});
  my @job_ids = $ats->submit_to_lsf({
    required_job_completion => $required_job_completion,
  });
  return @job_ids;
}

=head2 upload_illumina_analysis_to_qc_database

upload illumina analysis qc data 

=cut

sub upload_illumina_analysis_to_qc_database {
  my ($self, @args) = @_;
  my $required_job_completion = shift @args;
  my $aia = $self->new_with_cloned_attributes(q{npg_pipeline::archive::qc::illumina_analysis});
  my @job_ids = $aia->submit_to_lsf({
    required_job_completion => $required_job_completion,
  });
  return @job_ids;
}

=head2 upload_fastqcheck_to_qc_database

upload fastqcheck files for qc

=cut

sub upload_fastqcheck_to_qc_database {
  my ($self, @args) = @_;
  my $required_job_completion = shift @args;
  my $aia = $self->new_with_cloned_attributes(q{npg_pipeline::archive::qc::fastqcheck_loader});
  my @job_ids = $aia->submit_to_lsf({
    required_job_completion => $required_job_completion,
  });
  return @job_ids;
}

=head2 upload_summary_to_qc_database

This has been removed, since it is now loaded within upload_illumina_analysis_to_qc_database
code to be removed in release9.0

upload illumina summary data 

=cut

sub upload_summary_to_qc_database {
  my ($self, @args) = @_;
  my $required_job_completion = shift @args;
  my $asl = $self->new_with_cloned_attributes(q{npg_pipeline::archive::qc::summary_loader});
  my @job_ids = $asl->submit_to_lsf({
    required_job_completion => $required_job_completion,
  });
  return @job_ids;
}

=head2 upload_auto_qc_to_qc_database

upload internal auto_qc data

=cut

sub upload_auto_qc_to_qc_database {
  my ($self, @args) = @_;
  my $required_job_completion = shift @args;
  my $aaq = $self->new_with_cloned_attributes(q{npg_pipeline::archive::qc::auto_qc});
  my @job_ids = $aaq->submit_to_lsf({
    required_job_completion => $required_job_completion,
  });
  return @job_ids;
}

=head2 qc_db_delete_autoqc

removes autoqc results for a run (if they exist) except the contamination and qX yield results

=cut

sub qc_db_delete_autoqc {
  my ($self, @args) = @_;
  my $arg_refs = {
    required_job_completion => shift @args,
  };
  return $self->new_with_cloned_attributes( q{npg_pipeline::archive::mpsa_to_irods_migration_helper} )->launch_bsub_job($arg_refs);
}

=head2 flag_mpsa_migration_finished

marks run as having finished migration from mpsa to irods

=cut

sub flag_mpsa_migration_finished {
  my ($self, @args) = @_;
  my $arg_refs = {
    required_job_completion => shift @args,
  };
  return $self->new_with_cloned_attributes( q{npg_pipeline::archive::mpsa_to_irods_migration_helper} )->launch_bsub_job($arg_refs);
}


no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item Readonly

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
