#############
# $Id: pre_exec_strings.pm 13905 2011-08-15 09:52:41Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 
# Last Changed On: $Date: 2011-08-15 10:52:41 +0100 (Mon, 15 Aug 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-29.2/lib/npg_pipeline/roles/business/pre_exec_strings.pm $

package npg_pipeline::roles::business::pre_exec_strings;
use Moose::Role;

use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 13905 $ =~ /(\d+)/mxs; $r; };

=head1 NAME

npg_pipeline::roles::business::pre_exec_strings

=head1 VERSION

$LastChangedRevision: 13905 $

=head1 SYNOPSIS

  package My::Class;
  use Moose;
  with qw{npg_pipeline::roles::business::pre_exec_strings};

  ...
  no Moose;
  1;

=head1 DESCRIPTION

This roles exports into the class methods which generate pre-exec strings. Listed with the methods are the required methods
that those methods require.

=head1 SUBROUTINES/METHODS

=head2 ref_adapter_pre_exec_string

Method which returns the string to add to a bsub job which makes the script which runs this module
act as a pre-exec script in a bsub job.

This requires you to have at least the following methods

  'internal_script_bin'

Optionally, you can have both of the following

  'has_ref_repository'
  'ref_repository'

=cut

sub ref_adapter_pre_exec_string {
  my ( $self ) = @_;

  my $string = q{-E '} . $self->internal_script_bin() . q{/npg_pipeline_preexec_references};

  if ( $self->can( q{has_repository} ) && $self->has_repository() ) {
    $string .= q{ --repository } . $self->repository();
  }

  $string .= q{'};

  return $string;
}

=head2 files_present_pre_exec_string

pre-exec string to be used if files are to be tested for presence of

=cut

sub files_present_pre_exec_string {
  my ( $self, @files ) = @_;

  my $id_run = $self->id_run();

  my $start_command = q{-E '} . $self->internal_script_bin() .qq{/npg_pipeline_files_present --id_run $id_run};

  my $join_string = q{ --files };

  if ( ! scalar @files ) {

    my @lanes = $self->positions();

    my $archive_path = $self->runfolder_path() . q{/Latest_Summary/archive};
    if ( $self->no_summary_link() ) {
      $archive_path = $self->archive_path();
    }
    $join_string .= $archive_path . q{/completed_lane_};

    return qq{$start_command$join_string} . ( join $join_string, @lanes ) . q{'};
  }

  return qq{$start_command$join_string} . ( join $join_string, @files ) . q{'};
}

1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose::Role

=item Carp

=item English -no_match_vars

=item Readonly

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
