# $Id: 10-pluggable_harold_pb_cal_bam.t 13905 2011-08-15 09:52:41Z ajb $
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 8;
use Test::Exception::LessClever;
use Test::Deep;
use lib qw{t};
use t::util;
use File::Slurp;

my $util = t::util->new();
$util->catch_email();
my $conf_path = $util->conf_path();

$ENV{TEST_BIN} = q{t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();
$ENV{TEST_FS_RESOURCE} = q{nfs_12};

BEGIN {
  use_ok(q{npg_pipeline::pluggable::harold::PB_cal_bam});
}

my $runfolder_path = $util->analysis_runfolder_path();
$ENV{TEST_USERAGENT} = $runfolder_path . q{/useragent};

{
  $util->set_staging_analysis_area();
  write_file( $ENV{TEST_USERAGENT}, $util->serialized_useragent() );
  my $pb_cal_pipeline;
  lives_ok {
    $pb_cal_pipeline = npg_pipeline::pluggable::harold::PB_cal_bam->new({
      script_name => q{test},
      id_run => 1234,
      run_folder => q{123456_IL2_1234},
      conf_path => $conf_path,
      domain => q{test},
    });
  } q{no croak creating new object};
  isa_ok($pb_cal_pipeline, q{npg_pipeline::pluggable::harold::PB_cal_bam}, q{$pb_cal_pipeline});

  my $function_order = [ qw{
      make_multiplex_lane_specific_tag_files
      illumina2bam
      harold_alignment_files
      harold_calibration_tables
      harold_recalibration
      finish
  }];
  is_deeply( $pb_cal_pipeline->_default_function_order() , $function_order, q{Function order correct} );
#  lives_ok {
#    $pb_cal_pipeline->main();
#  } q{no croak processing through default pb_cal order};
}

{
  my $pb_cal_pipeline;
  lives_ok {
    $pb_cal_pipeline = npg_pipeline::pluggable::harold::PB_cal_bam->new({
      script_name => q{test},
      id_run => 1234,
      run_folder => q{123456_IL2_1234},
      conf_path => $conf_path,
      domain => q{test},
      no_recalibration => 1,
    });
  } q{no croak creating new object};

  ok( !scalar $pb_cal_pipeline->harold_alignment_files(),     q{no_recalibration - no alignment files launched} );
  ok( !scalar $pb_cal_pipeline->harold_calibration_tables(),  q{no_recalibration - no calibration tables launched} );
  ok( !scalar $pb_cal_pipeline->harold_recalibration(),       q{no_recalibration - no recalibration launched} );

}
1;
