# $Id: 20-archive_file_generation-bam.t 13905 2011-08-15 09:52:41Z ajb $
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 24;
use Test::Exception;
use lib qw{t};
use t::util;
use File::Temp qw/ tempfile tempdir /;

use_ok('npg_pipeline::archive::file::generation::bam');

my $util = t::util->new();
my $conf_path = $util->conf_path();

$ENV{TEST_BIN} = q{t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();

$ENV{TEST_RUN_OBJECT} = npg::api::run->new({
    util => $util->apiutil(),
    id_run => 1234,
  });

{
  $util->create_analysis( { no_touch_files => 1, bam_generation => 1, } );


  my $bam_generator;

  lives_ok { $bam_generator = npg_pipeline::archive::file::generation::bam->new({
    run_folder => q{123456_IL2_1234},
    timestamp => q{20090709-123456},
    verbose => 0,
    conf_path => $conf_path,
    domain => q{test},
    repository => q{t/data/sequence},
  }); } q{no croak creating $fastq_generator object};
  isa_ok($bam_generator, q{npg_pipeline::archive::file::generation::bam}, q{$bam_generator});

  ok( ! $bam_generator->_is_custom_qseq_file_original(1), 'lane 1 qseq file is not original' );
  ok( $bam_generator->_is_custom_qseq_file_original(2), 'lane 2 qseq file is original' ) ;
  ok( $bam_generator->_is_custom_qseq_file_original(3), 'lane 3 original qseq files not exist' ) ;

  my $arg_refs = {
    position => 1,
    fastq_files => ['1234_1_1.fastq', '1234_1_2.fastq'],
    required_job_completion => q{-w'done(123) && done(321)'},
  };

  my $hex;
  my $md5_object = Digest::MD5->new();
  my @bsub_commands = $bam_generator->_generate_bsub_commands({
      positions => [1,2,3,6,7,8],
      multiplex => 0,
      split_nc => 1,
      arg_refs => $arg_refs,
      lanes_by_type => {recalibrated_lanes => [1, 3], non_recalibrated_lanes  =>[2,6,7,8], },
      repository => q{t/data/sequence},
  });
  is( scalar @bsub_commands, 4, q{4 bsub commands returned} );
  my $bsub_command = $util->drop_temp_part_from_paths( $bsub_commands[0] );
  my $expected_command = q{bsub -q test -E 't/bin/software/solexa/bin/npg_pipeline_preexec_references --repository t/data/sequence' -R 'select[mem>8000] rusage[mem=8000,nfs_12=4] span[hosts=1]' -M8000000 -n4 -w'done(123) && done(321)' -J 'fastq2bam.pl_1234_20090709-123456[1,3]%64' -o /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/log/fastq2bam.pl_1234_20090709-123456.%I.%J.out 't/bin/software/solexa/bin/fastq2bam.pl --id_run 1234 --position `echo $LSB_JOBINDEX` --bam_filename /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`_human --reference  --reference_fai .fai --human_split human --original_quality_qseq_dir /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto --archive_path /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive --fastq_files /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`_human.fastq'};
  is( $bsub_command, $expected_command, q{first generated bsub command is correct});
  $bsub_command = $util->drop_temp_part_from_paths( $bsub_commands[1] );
  $expected_command = q{bsub -q test -E 't/bin/software/solexa/bin/npg_pipeline_preexec_references --repository t/data/sequence' -R 'select[mem>8000] rusage[mem=8000,nfs_12=4] span[hosts=1]' -M8000000 -n4 -w'done(123) && done(321)' -J 'fastq2bam.pl_1234_20090709-123456[1,3]%64' -o /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/log/fastq2bam.pl_1234_20090709-123456.%I.%J.out 't/bin/software/solexa/bin/fastq2bam.pl --id_run 1234 --position `echo $LSB_JOBINDEX` --bam_filename /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`_nonhuman --human_split nonhuman --original_quality_qseq_dir /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto --archive_path /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive --fastq_files /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`_nonhuman.fastq'};
  is( $bsub_command, $expected_command, q{second generated bsub command is correct});
  $bsub_command = $util->drop_temp_part_from_paths( $bsub_commands[2] );
  $expected_command = q{bsub -q test -E 't/bin/software/solexa/bin/npg_pipeline_preexec_references --repository t/data/sequence' -R 'select[mem>8000] rusage[mem=8000,nfs_12=4] span[hosts=1]' -M8000000 -n4 -w'done(123) && done(321)' -J 'fastq2bam.pl_1234_20090709-123456[2,6-8]%64' -o /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/log/fastq2bam.pl_1234_20090709-123456.%I.%J.out 't/bin/software/solexa/bin/fastq2bam.pl --id_run 1234 --position `echo $LSB_JOBINDEX` --bam_filename /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`_human --reference  --reference_fai .fai --human_split human --archive_path /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive --fastq_files /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`_human.fastq'};
  is( $bsub_command, $expected_command, q{third generated bsub command is correct});
  $bsub_command = $util->drop_temp_part_from_paths( $bsub_commands[3] );
  $expected_command = q{bsub -q test -E 't/bin/software/solexa/bin/npg_pipeline_preexec_references --repository t/data/sequence' -R 'select[mem>8000] rusage[mem=8000,nfs_12=4] span[hosts=1]' -M8000000 -n4 -w'done(123) && done(321)' -J 'fastq2bam.pl_1234_20090709-123456[2,6-8]%64' -o /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/log/fastq2bam.pl_1234_20090709-123456.%I.%J.out 't/bin/software/solexa/bin/fastq2bam.pl --id_run 1234 --position `echo $LSB_JOBINDEX` --bam_filename /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`_nonhuman --human_split nonhuman --archive_path /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive --fastq_files /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`_nonhuman.fastq'};
  is( $bsub_command, $expected_command, q{fourth generated bsub command is correct});

  $ENV{TEST_API_UTIL} = $util->apiutil();
  
  my @jids;
  lives_ok { @jids = $bam_generator->generate($arg_refs); } q{no croak running generate};
  is(scalar@jids, 5, q{5 job ids returned});

  #test spiked_phix jobs 
  @bsub_commands = $bam_generator->_generate_bsub_commands({
      positions => [1,2,3,6,7,8],
      multiplex => 0,
      split_nc => 0,
      spiked_phix => 1,
      arg_refs => $arg_refs,
      lanes_by_type => {recalibrated_lanes => [1, 3], non_recalibrated_lanes  =>[2,6,7,8], },
      repository => q{t/data/sequence},
  });
  is( scalar @bsub_commands, 2, q{2 spiked phix bam creation bsub commands returned} );
  $bsub_command = $util->drop_temp_part_from_paths( $bsub_commands[0] );
  $expected_command = q{bsub -q test -E 't/bin/software/solexa/bin/npg_pipeline_preexec_references --repository t/data/sequence' -R 'select[mem>8000] rusage[mem=8000,nfs_12=4] span[hosts=1]' -M8000000 -n4 -w'done(123) && done(321)' -J 'fastq2bam.pl_1234_20090709-123456[1,3]%64' -o /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/log/fastq2bam.pl_1234_20090709-123456.%I.%J.out 't/bin/software/solexa/bin/fastq2bam.pl --id_run 1234 --position `echo $LSB_JOBINDEX` --bam_filename /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`_phix --reference t/data/sequence/references/PhiX/default/all/bwa/phix-illumina.fa --reference_fai t/data/sequence/references/PhiX/default/all/fasta/phix-illumina.fa.fai --human_split phix --read_number_1 1 --original_quality_qseq_dir /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto --archive_path /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive --fastq_files /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`_phix.fastq'};
  is( $bsub_command, $expected_command, q{first spiked phix generated bsub command is correct} );
  $bsub_command = $util->drop_temp_part_from_paths( $bsub_commands[1] );
  $expected_command = q{bsub -q test -E 't/bin/software/solexa/bin/npg_pipeline_preexec_references --repository t/data/sequence' -R 'select[mem>8000] rusage[mem=8000,nfs_12=4] span[hosts=1]' -M8000000 -n4 -w'done(123) && done(321)' -J 'fastq2bam.pl_1234_20090709-123456[2,6-8]%64' -o /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/log/fastq2bam.pl_1234_20090709-123456.%I.%J.out 't/bin/software/solexa/bin/fastq2bam.pl --id_run 1234 --position `echo $LSB_JOBINDEX` --bam_filename /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`_phix --reference t/data/sequence/references/PhiX/default/all/bwa/phix-illumina.fa --reference_fai t/data/sequence/references/PhiX/default/all/fasta/phix-illumina.fa.fai --human_split phix --read_number_1 1 --archive_path /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive --fastq_files /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`_phix.fastq'};
  is( $bsub_command, $expected_command, q{second spiked generated bsub command is correct} );
}

{ 
  $util->create_multiplex_analysis( { bam_generation => 1, } );
  my $second_calls_dir = $util->standard_analysis_bustard_path() . q{/SecondCall};
  qx{mkdir -p $second_calls_dir};
  my $generator;

  lives_ok { $generator = npg_pipeline::archive::file::generation::bam->new({
    run_folder => q{123456_IL2_1234},
    timestamp => q{20090709-123456},
    verbose => 1,
    conf_path => $conf_path,
    domain => q{test},
    repository => q{t/data/sequence},
  }); } q{no croak creating $generator object};
  isa_ok($generator, q{npg_pipeline::archive::file::generation::bam}, q{$generator});

  my $dir = $generator->archive_path();
  my $runfolder = $generator->runfolder_path();
  mkdir "$dir/lane7";
  `touch $dir/lane7/1234_7_1_nonhuman#0.fastq`;
  `touch $dir/lane7/1234_7_2_nonhuman#0.fastq`;
  `touch $dir/lane7/1234_7_1_human#12.fastq`;
  `touch $dir/lane7/1234_7_2_human#12.fastq`;
  `touch $dir/lane7/1234_7_1#2.fastq`;
  `touch $dir/lane7/1234_7_2#2.fastq`;
  `cp t/data/tag_files/lane_7.tag $runfolder`;
  mkdir "$dir/lane8";
  `touch $runfolder/lane_8.tag`;
  my $arg_refs = {};
  $arg_refs->{required_job_completion}  = q{-w'done(123) && done(321)'};
  my @jids;
  lives_ok { @jids = $generator->generate($arg_refs); } q{no croak running generate};
  is(scalar@jids, 5, q{5 job ids returned});
  `cp t/data/tag_files/lane_7.tag $runfolder/lane_1.tag`;
  lives_ok { @jids = $generator->generate($arg_refs); } q{no croak running generate};
  is(scalar@jids, 5, q{5 job ids returned});
 
  my $cache = $generator->_lane_phix_splitting_cache();
  foreach my $lane (1..8) {
    $cache->{$lane} = q{true};
  }
  lives_ok { @jids = $generator->generate($arg_refs); } q{no croak running generate};
  is(scalar@jids, 6, q{6 job ids returned including phix splitting});

}
1;
