# $Id: 20-archive_file_generation-illumina2bam.t 13905 2011-08-15 09:52:41Z ajb $

use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 11;
use Test::Exception;
use lib qw{t};
use t::util;

use_ok('npg_pipeline::archive::file::generation::illumina2bam');

my $util = t::util->new();
my $conf_path = $util->conf_path();

$ENV{TEST_BIN} = q{t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();
local $ENV{NPG_WEBSERVICE_CACHE_DIR} = 't/data';

{
  $util->create_analysis({no_touch_files => 1});
  $util->copy_tag_file(8);

  my $bam_generator;

  lives_ok { $bam_generator = npg_pipeline::archive::file::generation::illumina2bam->new({
    run_folder => q{123456_IL2_1234},
    timestamp => q{20090709-123456},
    verbose => 0,
    conf_path => $conf_path,
    domain => q{test},
    id_run => 1234,
  }); } q{no croak creating bam_generator object};
  isa_ok($bam_generator, q{npg_pipeline::archive::file::generation::illumina2bam}, q{$bam_generator});

  isa_ok($bam_generator->lims, 'st::api::lims', 'cached lims object');
  is($bam_generator->lims->batch_id, 2015, 'batch id from lims object');

  my $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'}, 
  };
  
  my $alims = $bam_generator->lims->associated_child_lims_ia;
  my $position = 8;
  my $bsub_command = $bam_generator->_generate_bsub_commands( $arg_refs , $alims->{$position});

  $bsub_command = $util->drop_temp_part_from_paths( $bsub_command );
  my $expected_cmd = q{bsub -q test -R 'select[mem>2500] rusage[mem=2500,nfs_12=1]' -M2500000 -w'done(123) && done(321)' -J 'illumina2bam_1234_8_20090709-123456' -o /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/BaseCalls/log/illumina2bam_1234_8_20090709-123456.%J.out '/software/bin/java -Xms512m -Xmx1024m -jar /software/solexa/bin/aligners/illumina2bam/current/illumina2bam.jar CREATE_MD5_FILE=true I=/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto L=8 LIBRARY_NAME="51021" SAMPLE_ALIAS="NA18907" STUDY_NAME="1000Genomes Project Pilot 1: 1000Genomes Project Pilot 1" O=/dev/stdout COMPRESSION_LEVEL=0 | /software/bin/java -Xms512m -Xmx1024m -jar /software/solexa/bin/aligners/illumina2bam/current/BamIndexDecoder.jar CREATE_MD5_FILE=true VALIDATION_STRINGENCY=SILENT I=/dev/stdin  OUTPUT=/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/BaseCalls/1234_8.bam BARCODE_FILE=/nfs/sf45/IL2/analysis/123456_IL2_1234/lane_8.taglist METRICS_FILE=/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/BaseCalls/1234_8.bam.metrics'};
  
  is($bsub_command, $expected_cmd, 'correct bsub command for lane 8');
  
  my @jids;
  lives_ok { @jids = $bam_generator->generate($arg_refs); } q{no croak running generate};
  is(scalar @jids, 8, 'correct number of jobs submitted');  
  
  my $names = $bam_generator->_get_library_sample_study_names($alims->{1});
  is( $names->{study}, '1000Genomes Project Pilot 1: 1000Genomes Project Pilot 1', 'correct study name for lane 1');
  is( $names->{library}, '51021', 'correct library id for lane 1');
  is( $names->{sample}, 'NA18907', 'correct sample name for lane 1');
  
}

1;
__END__
