# $Id: 21-files_present.t 13905 2011-08-15 09:52:41Z ajb $
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 6;
use Test::Exception;
use lib q{t};
use t::util;

BEGIN {
  use_ok( q{npg_pipeline::pre_exec::FilesPresent} );
}

my $util = t::util->new();

my $conf_path = $util->conf_path();

my $runfolder = $util->default_runfolder();
$util->create_analysis();
my $runfolder_path = $util->analysis_runfolder_path();
my $recalibrated_path = $util->standard_analysis_recalibrated_path();

{

  my $files_present;
  lives_ok {
    $files_present = npg_pipeline::pre_exec::FilesPresent->new(
      id_run => 1234,
      run_folder => $runfolder,
      recalibrated_path => $recalibrated_path,
      lanes => [1,2,5,7,8],
      conf_path => $conf_path,
      domain => q{test},
    );
  } q{object ok};

  my $command_to_test = qq{bin/npg_pipeline_files_present --id_run 1234 --files $runfolder_path/Latest_Summary/archive/completed_lane_1 --files $runfolder_path/Latest_Summary/archive/completed_lane_2 --files $runfolder_path/Latest_Summary/archive/completed_lane_5 --files $runfolder_path/Latest_Summary/archive/completed_lane_7 --files $runfolder_path/Latest_Summary/archive/completed_lane_8};
  my $expected_command = qq{-E 't/bin/software/solexa/$command_to_test'};

  my $result_command = $files_present->files_present_pre_exec_string();
  is( $result_command, $expected_command, q{preexec command is correct} );

  qx{touch $recalibrated_path/archive/completed_lane_1};
  qx{touch $recalibrated_path/archive/completed_lane_2};
  qx{touch $recalibrated_path/archive/completed_lane_5};

  my $output = qx{$command_to_test};
  ok( $CHILD_ERROR, qq{Return code of $CHILD_ERROR as not all files found} );

  qx{touch $recalibrated_path/archive/completed_lane_7};
  qx{touch $recalibrated_path/archive/completed_lane_8};

  $output = qx{$command_to_test};
  ok( ! $CHILD_ERROR, qq{Return code of $CHILD_ERROR as all files found} );

  $result_command = $files_present->files_present_pre_exec_string(
    qw{this is a group of files},
  );

  $expected_command = q{-E 't/bin/software/solexa/bin/npg_pipeline_files_present --id_run 1234 --files this --files is --files a --files group --files of --files files'};

  is( $result_command, $expected_command, q{provided files works} );

}

1;