# $Id: 25-harold_calibration_bam.t 13905 2011-08-15 09:52:41Z ajb $
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 38;
use Test::Exception::LessClever;
use lib qw{t};
use t::util;
use t::useragent;
use DateTime;
use Test::MockObject;
use Cwd;
use File::Slurp;
use npg::api::run;

my $ua = t::useragent->new({ is_success => 1 });
my $util = t::util->new({
  useragent => $ua,
});
$util->catch_email();
my $api_run_object = npg::api::run->new({
  util => $util->apiutil(),
  id_run => 1234,
});

my $curdir = getcwd();

$ENV{TEST_API_UTIL} = $util->apiutil();
$ENV{TEST_BIN} = $curdir . q{/t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();
$ENV{TEST_FS_RESOURCE} = q{nfs_12};
$ENV{NPG_WEBSERVICE_CACHE_DIR} = $curdir . q{/t/data};

my ($mock_ref_finder, $id_run, $position, $refs, $single_ref_found);

BEGIN {
  $mock_ref_finder = Test::MockObject->new();
  $mock_ref_finder->set_bound( q{position}, \$position );
  $mock_ref_finder->set_bound( q{id_run}, \$id_run );
  $mock_ref_finder->set_bound( q{refs}, \$refs );
  $mock_ref_finder->set_bound( q{single_ref_found}, \$single_ref_found );
  $mock_ref_finder->fake_module( qw{npg_common::sequence::reference} );
  $mock_ref_finder->fake_new( qw{npg_common::sequence::reference} );
  
  use_ok(q{npg_pipeline::analysis::harold_calibration_bam});
}

my $conf_path = $util->conf_path();

my $runfolder_path = $util->analysis_runfolder_path();
my $bustard_home   = qq{$runfolder_path/Data/Intensities};
my $bustard_rta    = qq{$bustard_home/Bustard_RTA};
my $gerald_rta     = qq{$bustard_rta/GERALD_RTA};
my $config_path    = qq{$runfolder_path/Config};

$ENV{TEST_USERAGENT} = $runfolder_path . q{/useragent};

sub set_staging_analysis_area {
  `rm -rf /tmp/nfs/sf45`;
  `mkdir -p $bustard_rta`;
  `mkdir $config_path`;
  `cp t/data/Recipes/Recipe_GA2_37Cycle_PE_v6.1.xml $runfolder_path/`;
  `cp t/data/Recipes/TileLayout.xml $config_path/`;
  return 1;
}

{
  set_staging_analysis_area();
  my $harold;
  lives_ok {
    $harold = npg_pipeline::analysis::harold_calibration_bam->new({
      id_run => 1234,
      run_folder => q{123456_IL2_1234},
      timestamp => q{20091028-101635},
      lsf_queue => q{test},
      no_eland => 1,
      no_control_lane => 1,
      conf_path => $conf_path,
      domain => q{test},
      log_file_path => $runfolder_path,
      log_file_name => q{npg_pipeline_pb_cal_20091028-101635.log},
      instrument_model => q{HK},
      run => $api_run_object,
    });
  } q{create $harold object ok};
  isa_ok($harold, q{npg_pipeline::analysis::harold_calibration_bam}, q{$harold});
  my @job_ids = $harold->generate_calibration_table({});
  is( scalar @job_ids, 0, q{no job ids as no_eland and no_control_lane set} );
  is( $harold->find_bustard(), $bustard_rta, q{bustard dir found ok} );

  @job_ids = $harold->generate_recalibrated_qseqs({});
  is( scalar @job_ids, 0, q{no job ids as no_eland and no_control_lane set} );
  is( $harold->find_bustard(), $bustard_rta, q{bustard dir found ok} );

  is( $harold->calibration_table_name( { id_run => 1234, read => 2 } ),  q{}, q{generated calibration table name is empty string});

}

{
  set_staging_analysis_area();
  my $harold;
  lives_ok {
    $harold = npg_pipeline::analysis::harold_calibration_bam->new({
      id_run => 1234,
      run_folder => q{123456_IL2_1234},
      timestamp => q{20091028-101635},
      lsf_queue => q{test},
      verbose => 1,
      conf_path => $conf_path,
      domain => q{test},
      repository => $curdir . q{/t/data/sequence},
      instrument_model => q{HK},
    });
  } q{create $harold object ok};

  my $req_job_completion = q{-w'done(123) && done(321)'};
  my $arg_refs = {
    required_job_completion => $req_job_completion,
  };

  write_file( $ENV{TEST_USERAGENT}, $util->serialized_useragent() );

  my @job_ids;

  my $job = qq{bsub -q test -E 't/bin/software/solexa/bin/npg_pipeline_preexec_references --repository $curdir/t/data/sequence' -o log/PB_cal_table_1234_4_20091028-101635.out -J PB_cal_table_1234_4_20091028-101635 -C 99999999 -R 'select[mem>3072] rusage[mem=3072,nfs_12=4] span[hosts=1]' -M 3072000 -w'done(123) && done(321)' " t/bin/software/solexa/src/new-pipeline-dev/pb_calibration/branches/current/bin/pb_calibration --runfolder_dir $runfolder_path --intensity_dir $bustard_home --run 1234 --lane 4 --t_filter 2 --read 1 --cstart 1 --prefix 1234_4_1 --mode 2 --bam pb_align_1234_4.bam "};

  lives_ok {
    @job_ids = $harold->generate_calibration_table( $arg_refs );
  } q{no croak submitting calibration table jobs};
  is( scalar @job_ids, 2, q{2 jobs created});
  note $job_ids[0];

  is( $harold->_calibration_table_bsub_command( {
   dir => $bustard_rta,
   position => 4,
   read => 1,
   job_dependencies => $req_job_completion,
   ref_seq => q{phix-illumina.fa},
   control => 1,
  } ), $job, q{generated bsub command is correct} );

  my $cal_table = q{1234_4_2_train_purity_caltable.txt};
  is( $harold->calibration_table_name( { id_run => 1234, read => 2, mode => 2 } ), $cal_table, q{generated calibration table name is correct});
  $job = qq{bsub -q test -o log/PB_cal_score_1234_3_20091028-101635.out -J PB_cal_score_1234_3_20091028-101635 -C 99999999 -R 'select[mem>1725] rusage[mem=1725,nfs_12=4] span[hosts=1]' -M 1725000 -n 4 -w'done(123) && done(321)' " t/bin/software/solexa/src/new-pipeline-dev/pb_calibration/branches/current/bin/pb_predictor --bam ${bustard_home}/BaseCalls/1234_3.bam --prefix 1234_3 --run 1234 --cstart 1 --clmode 2 --ct1 1234_4_2_train_purity_caltable.txt --ct2 1234_4_2_train_purity_caltable.txt "};
  is( $harold->_recalibration_bsub_command( {
    position => 3,
    read => 1,
    job_dependencies => $req_job_completion,
    ct1 => $cal_table,
    ct2 => $cal_table,
  } ), $job, q{generated bsub command for recalibration job is correct});

  lives_ok {
    @job_ids = $harold->generate_recalibrated_qseqs($arg_refs);
  } q{no croak submitting recalibration jobs};
  is( scalar @job_ids, 8, q{8 jobs created});

}

{
  set_staging_analysis_area();
  my $harold;
  lives_ok {
    $harold = npg_pipeline::analysis::harold_calibration_bam->new({
      id_run => 1234,
      run_folder => q{123456_IL2_1234},
      timestamp => q{20091028-101635},
      lsf_queue => q{test},
      verbose => 1,
      preferred_control_lane => 4,
      conf_path => $conf_path,
      domain => q{test},
      repository => $curdir . q{/t/data/sequence},
#      instrument_model => q{HK},
    });
  } q{create $harold object ok};

  my $req_job_completion = q{-w'done(123) && done(321)'};
  my $arg_refs = {
    required_job_completion => $req_job_completion,
  };

  $ENV{TEST_RUN_OBJECT} = npg::api::run->new({
    util   => $util->apiutil(),
    id_run => 1234,
  });

  my @job_ids;

  my $job = qq{bsub -q test -E 't/bin/software/solexa/bin/npg_pipeline_preexec_references --repository $curdir/t/data/sequence' -o log/PB_cal_table_1234_4_20091028-101635.out -J PB_cal_table_1234_4_20091028-101635 -C 99999999 -R 'select[mem>3072] rusage[mem=3072,nfs_12=4] span[hosts=1]' -M 3072000 -w'done(123) && done(321)' " t/bin/software/solexa/src/new-pipeline-dev/pb_calibration/branches/current/bin/pb_calibration --runfolder_dir $runfolder_path --intensity_dir $bustard_home --run 1234 --lane 4 --t_filter 2 --read 1 --cstart 1 --prefix 1234_4_1 --mode 2 --bam pb_align_1234_4.bam "};

  is( $harold->_calibration_table_bsub_command( {
   position => 4,
   read => 1,
   job_dependencies => $req_job_completion,
   ref_seq => q{phix-illumina.fa},
   control => 1,
  } ), $job, q{generated cal_table control bsub command is correct} );

  lives_ok {
    @job_ids = $harold->generate_calibration_table( $arg_refs );
  } q{no croak submitting calibration table jobs};
  is( scalar @job_ids, 2, q{2 jobs created} );
  $job = qq{bsub -q test -o log/PB_cal_score_1234_4_20091028-101635.out -J PB_cal_score_1234_4_20091028-101635 -C 99999999 -R 'select[mem>1725] rusage[mem=1725,nfs_12=4] span[hosts=1]' -M 1725000 -n 4 -w'done(123) && done(321)' " t/bin/software/solexa/src/new-pipeline-dev/pb_calibration/branches/current/bin/pb_predictor --bam /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/BaseCalls/1234_4.bam --prefix 1234_4 --run 1234 --cstart 1 --clmode 2 --ct 1234_4_1_train_purity_cycle_caltable.txt "};

  is( $util->drop_temp_part_from_paths( $harold->_recalibration_bsub_command( {
    dir => $bustard_rta,
    position => 4,
    read => 1,
    job_dependencies => $req_job_completion,
    ct => q{1234_4_1_train_purity_cycle_caltable.txt},
  } ) ), $job, q{generated recalibration control bsub command is correct} );

  lives_ok {
    @job_ids = $harold->generate_recalibrated_qseqs( $arg_refs );
  } q{no croak submitting recalibration jobs};
  is( scalar @job_ids, 8, q{8 jobs created - all using control lane caltable} );

}

{
  set_staging_analysis_area();
  my $harold;
  $id_run = 4846;
  lives_ok {
    $harold = npg_pipeline::analysis::harold_calibration_bam->new({
      id_run => $id_run,
      run_folder => q{123456_IL2_1234},
      timestamp => q{20091028-101635},
      lsf_queue => q{test},
      no_control_lane => 1,
      verbose => 1,
      conf_path => $conf_path,
      domain => q{test},
      repository => $curdir . q{/t/data/sequence},
      instrument_model => q{HK},
    });
  } q{create $harold object ok};
  $refs = [];
  isa_ok($harold, q{npg_pipeline::analysis::harold_calibration_bam}, q{$harold});
  my @job_ids = $harold->generate_calibration_table({});
  is( scalar @job_ids, 0, q{no job ids as no_control_lane set and no references found} );
  $refs = [ qw{ test with more than one ref returned } ];
  @job_ids = $harold->generate_calibration_table({});
  is( scalar @job_ids, 0, q{no job ids as no_control_lane set, and multiple references found} );
  @job_ids = $harold->generate_recalibrated_qseqs({});
  is( scalar @job_ids, 0, q{no job ids as no_control_lane set, and no single reference able to be returned} );

  $single_ref_found = 1;
  $refs = [ q{/path/to/references/fasta.fa} ];

  @job_ids = $harold->generate_calibration_table({});
  is( scalar @job_ids, 8, q{8 job ids as no_control_lane set, and single references found} );


  @job_ids = $harold->generate_recalibrated_qseqs({});
  is( scalar @job_ids, 8, q{8 job ids as no_control_lane set, so lanes own cal tables used} );

}

{
  set_staging_analysis_area();
  my $harold;
  $id_run = 1234;
  lives_ok {
    $harold = npg_pipeline::analysis::harold_calibration_bam->new({
      id_run => $id_run,
      run_folder => q{123456_IL2_1234},
      timestamp => q{20091028-101635},
      lsf_queue => q{test},
      verbose => 1,
      conf_path => $conf_path,
      domain => q{test},
      repository => $curdir . q{/t/data/sequence},
      instrument_model => q{HK},

    });
  } q{create $harold object ok};


  my @job_ids;
  lives_ok {
    @job_ids = $harold->generate_calibration_table({});
  } q{got job_ids ok};
  is( scalar @job_ids, 8, q{8 job ids as no_control_lane set, and single references found} );


  @job_ids = $harold->generate_recalibrated_qseqs({});
  is( scalar @job_ids, 8, q{8 job ids as no_control_lane set, so lanes own cal tables used} );

  my $arg_refs = {
    timestamp => q{20091028-101635},
    position => 1,
    job_dependencies => q{-w 'done(1234) && done(4321)'},
    ref_seq => q{t/data/sequence/references/Human/default/all/bwa/someref.fa.bwt},
  };

  my $single_read_alignment_command = q{bsub -q test -E 't/bin/software/solexa/bin/npg_pipeline_preexec_references --repository t/data/sequence' -o log/PB_cal_align_1234_1_20091028-101635.out -J PB_cal_align_1234_1_20091028-101635 -C 99999999 -R 'select[mem>8000] rusage[mem=8000,nfs_12=4] span[hosts=1]' -M 8000000 -n 4 -w 'done(1234) && done(4321)' " t/bin/software/solexa/src/new-pipeline-dev/pb_calibration/branches/current/bin/pb_align --aln_parms '-t 4' --ref t/data/sequence/references/Human/default/all/bwa/someref.fa.bwt --read 1 --bam /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/BaseCalls/1234_1.bam --prefix pb_align_1234_1 --pf_filter "};
  my $paired_read_alignment_command = q{bsub -q test -E 't/bin/software/solexa/bin/npg_pipeline_preexec_references --repository t/data/sequence' -o log/PB_cal_align_1234_1_20091028-101635.out -J PB_cal_align_1234_1_20091028-101635 -C 99999999 -R 'select[mem>8000] rusage[mem=8000,nfs_12=4] span[hosts=1]' -M 8000000 -n 4 -w 'done(1234) && done(4321)' " t/bin/software/solexa/src/new-pipeline-dev/pb_calibration/branches/current/bin/pb_align --aln_parms '-t 4' --ref t/data/sequence/references/Human/default/all/bwa/someref.fa.bwt --read1 1 --read2 2 --bam /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/BaseCalls/1234_1.bam --prefix pb_align_1234_1 --pf_filter "};
  my $spiked_read_alignment_command = q{bsub -q test -E 't/bin/software/solexa/bin/npg_pipeline_preexec_references --repository t/data/sequence' -o log/PB_cal_align_1234_1_20091028-101635.out -J PB_cal_align_1234_1_20091028-101635 -C 99999999 -R 'select[mem>8000] rusage[mem=8000,nfs_12=4] span[hosts=1]' -M 8000000 -n 4 -w 'done(1234) && done(4321)' " t/bin/software/solexa/src/new-pipeline-dev/pb_calibration/branches/current/bin/pb_align --aln_parms '-t 4' --ref t/data/sequence/references/PhiX/default/all/fasta/phix-illumina.fa --read1 1 --read2 2 --bam /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/BaseCalls/1234_1.bam --prefix pb_align_1234_1 --pf_filter "};

  is( $util->drop_temp_part_from_paths( $harold->_alignment_file_bsub_command( $arg_refs ) ), $single_read_alignment_command, q{single read alignment bsub command is correct} );

  $arg_refs->{is_paired} = 1;
  is( $util->drop_temp_part_from_paths( $harold->_alignment_file_bsub_command( $arg_refs ) ), $paired_read_alignment_command, q{paired read alignment bsub command is correct} );

  $arg_refs->{is_spiked_phix} = 1;
  $arg_refs->{ref_seq} = q{t/data/sequence/references/PhiX/default/all/fasta/phix-illumina.fa};
  is( $util->drop_temp_part_from_paths( $harold->_alignment_file_bsub_command( $arg_refs ) ), $spiked_read_alignment_command, q{paired read alignment bsub command is correct} );

  @job_ids = $harold->generate_alignment_files({});
  is( scalar @job_ids, 8, q{8 job ids, no_control_lane set} );

}


1;
