# $Id: 50-archival_runner.t 13905 2011-08-15 09:52:41Z ajb $
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 8;
use Test::Exception;
use lib qw{t};
use t::util;
use npg::api::run_status_dict;


BEGIN {
  use_ok('npg_pipeline::daemons::archival_runner');
}

my $util = t::util->new();
my $conf_path = $util->conf_path();

my $temp_directory = $util->temp_directory();

$ENV{TEST_BIN} = q{t/bin};
$ENV{TEST_LOG} = $temp_directory;

`rm -rf $temp_directory/software/solexa/logs; mkdir -p $temp_directory/software/solexa/logs`;

{
  my $runner;
  lives_ok { $runner = npg_pipeline::daemons::archival_runner->new({
    conf_path => $conf_path,
    domain => q{test},
  }); } q{object creation ok};
  isa_ok($runner, q{npg_pipeline::daemons::archival_runner}, q{$runner});
  $runner->_set_api_util($util->apiutil());
  lives_ok { $runner->run(); } q{no croak on $runner->run()};
  is($runner->_generate_command(1234), q{source t/bin/software/oracle/etc/profile.oracle ; source t/bin/software/lsf/conf/profile.lsf ; t/bin/software/solexa/bin/npg_pipeline_post_qc_review --verbose --id_run=1234}, q{generated command is correct});
  my $runs = npg::api::run_status_dict->new({
    description => q{archival pending},
    util => $runner->_api_util(),
  })->runs();
  is($runner->_use_lowest_id_run($runs->[2]), $runs->[2]->id_run(), q{chose lowest as id_run correctly});
  is($runner->_use_lowest_id_run($runs->[1]), $runs->[1]->id_run_pair(), q{chose lowest as id_run_pair correctly});
}
{
  my $runner = npg_pipeline::daemons::archival_runner->new({
    conf_path => $conf_path,
    domain => q{test},
  });
  $runner->_set_api_util($util->apiutil());
  lives_ok {
    my $count = 0;
    while ($count < 2) {
      eval {
        $runner->run();
        1;
      } or do {
        carp q{Error running archival runner: } . $EVAL_ERROR . q{ - Going to sleep};
      };
      sleep 2;
      $count++;
    }
  } q{no croak running through twice - potentially as a daemon process};

}

1;